"""
A test script for the presentation on nested lists and dictionaries

These tests are provided so that you can try these functions out yourself.

Author: Walker M. White
Date: October 30, 2020
"""
import nest
import hist
import introcs


def test_sum_columns():
    """
    Test procedure for the function sum_columns()
    """
    print('Testing sum_columns()')

    input = [[1]]
    result = nest.sum_columns(input)
    introcs.assert_equals([1],result)
    # Always test input unchanged
    introcs.assert_equals([[1]],input)

    input = [[1, 2], [3, 4]]
    result = nest.sum_columns(input)
    introcs.assert_equals([4, 6],result)
    # Always test input unchanged
    introcs.assert_equals([[1, 2], [3, 4]],input)

    input = [[1, 2], [3, 4], [5, 6]]
    result = nest.sum_columns(input)
    introcs.assert_equals([9, 12],result)
    # Always test input unchanged
    introcs.assert_equals([[1, 2], [3, 4], [5, 6]],input)

    input = [[1, 2, 3], [4, 5, 6]]
    result = nest.sum_columns(input)
    introcs.assert_equals([5, 7, 9],result)
    # Always test input unchanged
    introcs.assert_equals([[1, 2, 3], [4, 5, 6]],input)

    input = [[1, 2, 3]]
    result = nest.sum_columns(input)
    introcs.assert_equals([1, 2, 3],result)
    # Always test input unchanged
    introcs.assert_equals([[1, 2, 3]],input)


def test_tablize():
    """
    Test procedure for the function tablize()
    """
    print('Testing tablize()')

    input = [[1]]
    result = nest.tablize(input)
    introcs.assert_equals([[1]],input)
    # Always verify not fruitful
    introcs.assert_equals(None,result)

    input = [[1], [2,3]]
    result = nest.tablize(input)
    introcs.assert_equals([[1,0],[2,3]],input)
    # Always verify not fruitful
    introcs.assert_equals(None,result)

    input = [[1,2], [3,4,5], [6,7]]
    result = nest.tablize(input)
    introcs.assert_equals([[1,2,0],[3,4,5],[6,7,0]],input)
    # Always verify not fruitful
    introcs.assert_equals(None,result)

    input = [[1,2], [3,4], [5,6,7]]
    result = nest.tablize(input)
    introcs.assert_equals([[1,2,0],[3,4,0],[5,6,7]],input)
    # Always verify not fruitful
    introcs.assert_equals(None,result)

    input = [[1,2], [3,4], [5,6]]
    result = nest.tablize(input)
    introcs.assert_equals([[1,2],[3,4],[5,6]],input)
    # Always verify not fruitful
    introcs.assert_equals(None,result)


def test_merge():
    """
    Test procedure for the function merge()
    """
    print('Testing merge()')

    input1 = { 'a':1 }
    input2 = { 'b':2 }
    result = hist.merge(input1,input2)
    introcs.assert_equals({'a':1, 'b':2 }, result)
    # Always test input unchanged
    introcs.assert_equals(input1, { 'a':1 })
    introcs.assert_equals(input2, { 'b':2 })

    input1 = { 'a':1 }
    input2 = { 'a':1,'b':2 }
    result = hist.merge(input1,input2)
    introcs.assert_equals({'a':2, 'b':2 }, result)
    # Always test input unchanged
    introcs.assert_equals(input1, { 'a':1 })
    introcs.assert_equals(input2, { 'a':1,'b':2 })

    input1 = { 'a':1,'b':2 }
    input2 = { 'a':1 }
    result = hist.merge(input1,input2)
    introcs.assert_equals({'a':2, 'b':2 }, result)
    # Always test input unchanged
    introcs.assert_equals(input1, { 'a':1,'b':2 })
    introcs.assert_equals(input2, { 'a':1 })


    input1 = { 'a':1,'b':2, 'c':3, 'd':5 }
    input2 = { 'a':1,'b':4, 'c':2, 'e':4 }
    result = hist.merge(input1,input2)
    introcs.assert_equals({ 'a':2,'b':6, 'c':5, 'd':5, 'e':4 }, result)
    # Always test input unchanged
    introcs.assert_equals(input1, { 'a':1,'b':2, 'c':3, 'd':5 })
    introcs.assert_equals(input2, { 'a':1,'b':4, 'c':2, 'e':4 })

    input1 = { 'a':1,'b':2 }
    input2 = { }
    result = hist.merge(input1,input2)
    introcs.assert_equals({'a':1, 'b':2 }, result)
    # Always test input unchanged
    introcs.assert_equals(input1, { 'a':1,'b':2 })
    introcs.assert_equals(input2, { })

    input1 = { }
    input2 = { 'a':1,'b':2 }
    result = hist.merge(input1,input2)
    introcs.assert_equals({'a':1, 'b':2 }, result)
    # Always test input unchanged
    introcs.assert_equals(input1, { })
    introcs.assert_equals(input2, { 'a':1,'b':2 })

    input1 = { }
    input2 = { }
    result = hist.merge(input1,input2)
    introcs.assert_equals({ }, result)
    # Always test input unchanged
    introcs.assert_equals(input1, { })
    introcs.assert_equals(input2, { })


def test_histogram1():
    """
    Test procedure for the function histogram1()
    """
    print('Testing histogram1()')

    example = { 'qsq4':80, 'dju6':90, 'wgo5':75, 'zkw9':60, 'wj8':74}

    result = hist.histogram1(example,90,75)
    introcs.assert_equals({'A':1, 'B':2, 'C':2},result)
    # Always test input unchanged
    introcs.assert_equals(example, { 'qsq4':80, 'dju6':90, 'wgo5':75, 'zkw9':60, 'wj8':74})

    result = hist.histogram1(example,80,60)
    introcs.assert_equals({'A':2, 'B':3, 'C':0},result)
    # Always test input unchanged
    introcs.assert_equals(example, { 'qsq4':80, 'dju6':90, 'wgo5':75, 'zkw9':60, 'wj8':74})

    result = hist.histogram1(example,70,50)
    introcs.assert_equals({'A':4, 'B':1, 'C':0},result)
    # Always test input unchanged
    introcs.assert_equals(example, { 'qsq4':80, 'dju6':90, 'wgo5':75, 'zkw9':60, 'wj8':74})

    example = {}
    result = hist.histogram1(example,90,75)
    introcs.assert_equals({'A':0, 'B':0, 'C':0},result)
    # Always test input unchanged
    introcs.assert_equals(example, { })


def test_histogram2():
    """
    Test procedure for the function histogram2()
    """
    print('Testing histogram2()')

    result = hist.histogram2('')
    introcs.assert_equals({},result)

    result = hist.histogram2('a')
    introcs.assert_equals({'a':1},result)

    result = hist.histogram2('abc')
    introcs.assert_equals({'a':1,'b':1,'c':1},result)

    result = hist.histogram2('all')
    introcs.assert_equals({'a':1,'l':2},result)

    result = hist.histogram2('abracadabra')
    introcs.assert_equals({'a':5,'b':2,'c':1,'d':1,'r':2},result)


if __name__ == '__main__':
    test_sum_columns()
    test_tablize()
    test_merge()
    #test_histogram1()
    test_histogram2()
    print('The list and dictionary functions passed all tests.')
